import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
import graphics.graphics as graphics
import graphics.animation as animation
from . import channels as bgChannels
from . import collisions as coll
from .types import *
import time

walkVel     = 300.0
gravity     = 2000.0
jumpVel     = -600.0
dblJumpVel  = -600.0

speedUp     = 100.0
jumpUp      = -200.0
powerTime   = 10.0

catStartX = 50.0
catStartY = 450.0

rectWidth   = 53.0
rectHeight  = 37.0
spriteWidth = 64.0
spriteHeight = 64.0
rectOffsetX = 5.0
rectOffsetY = -5.0

maxChewTime = 1.2

idleAnimDesc = (dataPath + "mochicat/idle/mochiIdle", 0, 1.0)
walkAnimDesc = (dataPath + "mochicat/walk/mochiWalk", 3, 0.1)
jumpAnimDesc = (dataPath + "mochicat/jump/mochiJump", 1, 0.2)
fallAnimDesc = (dataPath + "mochicat/fall/mochiFall", 1, 0.2)
hurtAnimDesc = (dataPath + "mochicat/hurt/mochiHurt", 0, 0.2)
chewAnimDesc = (dataPath + "mochicat/chew/mochiChew", 3, 0.15)
chewWalkAnimDesc = (dataPath + "mochicat/chewwalk/mochiChewWalk", 3, 0.15)
chewJumpAnimDesc = (dataPath + "mochicat/chew/mochiJumpChew", 1, 0.2)
chewFallAnimDesc = (dataPath + "mochicat/chew/mochiFallChew", 1, 0.2)

class Cat:
    def __init__(self, goal):
        self.x = catStartX
        self.y = catStartY
        self.direction = Direction.RIGHT
        self.scale = 1
        self.rect = self.generateRect(catStartX, catStartY)
        self.velX = 0.0
        self.velY = 0.0
        self.walkVel = walkVel
        self.jumpVel = jumpVel
        self.dblJumpVel = dblJumpVel
        self.onGround = False
        self.doubleJumped = False
        self.goal = goal
        self.eaten = 0
        self.power = []
        self.resetEaten = 0
        self.resetGoal = 1
        self.chewTime = 0.0

        self.idleAnim = animation.Animation(idleAnimDesc, True)
        self.walkAnim = animation.Animation(walkAnimDesc, True)
        self.jumpAnim = animation.Animation(jumpAnimDesc, False)
        self.fallAnim = animation.Animation(fallAnimDesc, False)
        self.hurtAnim = animation.Animation(hurtAnimDesc, False)
        self.chewAnim = animation.Animation(chewAnimDesc, True)
        self.chewWalkAnim = animation.Animation(chewWalkAnimDesc, True)
        self.chewJumpAnim = animation.Animation(chewJumpAnimDesc, False)
        self.chewFallAnim = animation.Animation(chewFallAnimDesc, False)
        self.anim = self.idleAnim

    def draw(self):
        #graphics.drawRect(g.window, self.rect, sf.Color.Magenta)

        self.anim.setFlipped(self.direction == Direction.RIGHT)
        self.anim.setScale(self.scale)
        self.anim.draw(self.x, self.y - self.scale * spriteHeight / 2, True)

    def generateRect(self, x, y):
        return sf.FloatRect(x - self.scale * rectWidth / 2,
                            y - self.scale * rectHeight,
                            x + self.scale * rectWidth / 2,
                            y)

    def move(self, x, y):
        if x < self.scale * rectWidth / 2:
            x = self.scale * rectWidth / 2
        elif x > windowWidth - self.scale * rectWidth / 2:
            x = windowWidth - self.scale * rectWidth / 2
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def reset(self, goal):
        self.resetGoal = self.goal
        self.goal = goal
        self.resetEaten = self.eaten
        self.eaten = 0
        self.resize([])

    def resize(self, eatenList):
        amountEaten = 0
        eat = False
        for f in eatenList:
            eat = True
            if (f.type == FoodType.POINT):
                amountEaten += f.value
                if f.value < 0:
                    eat = False
                    if self.anim != self.hurtAnim:
                        self.anim = self.hurtAnim
                        self.anim.reset()
            else:
                self.power.append((f.type, powerTime))

            if (f.type == FoodType.SPEEDUP):
                self.walkVel += speedUp
            elif (f.type == FoodType.JUMPUP):
                self.jumpVel += jumpUp
                self.dblJumpVel += jumpUp

        if (self.eaten + amountEaten < -10):
            amountEaten = -10 - self.eaten

        self.eaten += amountEaten

        if eat:
            self.chewTime = maxChewTime

        if self.resetEaten > 0:
            self.resetEaten -= 1
        elif self.resetEaten < 0:
            self.resetEaten += 1

        self.scale = 1 + 3 * ((self.eaten / self.goal) + (self.resetEaten / self.resetGoal))

    def update(self, inputState):
        timeNow = time.time()

        temp = []
        for (ty, sec) in self.power:
            temp.append((ty, sec - g.timeDelta))

        finished = [ty for (ty, sec) in temp if sec <= 0]
        for ty in finished:
            if ty == FoodType.SPEEDUP:
                self.walkVel -= speedUp
            elif ty == FoodType.JUMPUP:
                self.jumpVel -= jumpUp
                self.dblJumpVel -= jumpUp

        self.power = [(ty, sec) for (ty, sec) in temp if sec > 0]

        # walk left/right
        self.velX = 0.0
        if inputState.keyDown("left") or inputState.joyLeft:
            self.velX = -self.walkVel
            self.direction = Direction.LEFT
        elif inputState.keyDown("right") or inputState.joyRight:
            self.velX = self.walkVel
            self.direction = Direction.RIGHT

        # jump
        jumpPressed = inputState.keyPressed("c") or inputState.joyPressed("jump")
        if jumpPressed and self.onGround:
            self.velY = self.jumpVel
        # double jump
        elif jumpPressed and not self.doubleJumped:
            self.doubleJumped = True
            self.velY = self.dblJumpVel

        # gravity
        self.velY += gravity * g.timeDelta

        # Check y bottom
        if (self.y > windowHeight):
            self.y = catStartY
            self.velY = 0

        # new position
        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta
        newRect = self.generateRect(newX, newY)

        # submit collision detection requests
        collisionInfoReq = [coll.CollisionInfo("cat", newRect), 
                            coll.CollisionRequest("catReq", newRect, ["floor", "wall"])]
        bgChannels.collisionsChannel.send(collisionInfoReq)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        self.onGround = False

        # process collisions
        collResp = collResponses["catReq"]
        for cr in collResp:
            if cr.intersects:

                # touching floor
                if cr.collType == "floor":
                    self.move(newX, newY)
                    self.unstickCollision(newX, newY, newRect, cr.collRect)
                    self.velY = 0.0
                    self.onGround = True
                    self.doubleJumped = False

                # touching wall
                elif cr.collType == "wall":
                    if not self.onGround:
                        self.move(newX, newY)
                    self.unstickCollision(newX, newY, newRect, cr.collRect)
            else:   
                self.move(newX, newY)

        self.updateAnimation()

    def updateAnimation(self):
        self.anim.update(g.timeDelta)

        if self.anim == self.hurtAnim:
            if not self.hurtAnim.finished:
                return

        if self.onGround:
            if self.chewTime > 0:
                self.chewTime -= g.timeDelta
                if self.velX == 0.0:
                    self.anim = self.chewAnim
                else:
                    self.anim = self.chewWalkAnim
            # idle
            elif self.velX == 0.0:
                self.anim = self.idleAnim
            # walk
            else:
                self.anim = self.walkAnim
        else:
            if self.chewTime > 0:
                if self.velY < 0.0:
                    if self.anim != self.chewJumpAnim:
                        self.anim = self.chewJumpAnim
                        self.anim.reset()
                else:
                    if self.anim != self.chewFallAnim:
                        self.anim = self.chewFallAnim
                        self.anim.reset()
            # jump
            elif self.velY < 0.0:
                if self.anim != self.jumpAnim:
                    self.anim = self.jumpAnim
                    self.anim.reset()
            # fall
            else:
                if self.anim != self.fallAnim:
                    self.anim = self.fallAnim
                    self.anim.reset()

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

    def info(self):
        return CatInfo((self.x, self.y), self.onGround, self.eaten)

def catTasklet():
    global cat
    
    try:
        goal = bgChannels.amountEatenChannel.receive()
        cat = Cat(goal)
        channels.broadcast(bgChannels.catInfoChannel, cat.info())

        while 1:
            inputState = channels.inputChannel.receive()

            third = goal / 3
            sun = 0
            if (cat.eaten < third):
                sun = 0
            elif cat.eaten < 2 * third:
                sun = 1
            else:
                sun = 2

            channels.broadcast(bgChannels.musicInputChannel, MusicInput(inputState, sun))

            cat.update(inputState)

            eatenList = bgChannels.amountEatenChannel.receive()
            
            channels.broadcast(bgChannels.catInfoChannel, cat.info())
            
            cat.draw()

            cat.resize(eatenList)
    except TaskletExit:
        cat.idleAnim.setScale(1.0)
        cat.walkAnim.setScale(1.0)
        cat.jumpAnim.setScale(1.0)
        cat.fallAnim.setScale(1.0)
        cat.hurtAnim.setScale(1.0)

def catResetTasklet():
    global cat
    
    while 1:
        goal = bgChannels.catResetChannel.receive()
        cat.reset(goal)
