from PySFML import sf
import channels.channels as channels

class InputState:
    class KeyState:
        def __init__(self, keyCode):
            self.code        = keyCode
            self.down        = False
            self.prevDown    = False

    class BtnState:
        def __init__(self, num):
            self.num        = num
            self.down       = False
            self.prevDown   = False

    def __init__(self, keysList, joyBtns):
        self.mouseX         = 0
        self.mouseY         = 0
        self.mouse0Down     = False
        self.prevMouse0Down = False
        self.joyLeft        = False
        self.joyRight       = False

        self.keys = dict()
        for (keyName, keyCode) in keysList:
            self.keys[keyName] = self.KeyState(keyCode)

        self.joyBtns = dict()
        for (btnName, btnNum) in joyBtns:
            self.joyBtns[btnName] = self.BtnState(btnNum)

    def keyDown(self, keyName):
        return self.keys[keyName].down

    def keyPressed(self, keyName):
        return not self.keys[keyName].prevDown and \
               self.keys[keyName].down

    def joyDown(self, btnName):
        return self.joyBtns[btnName].down

    def joyPressed(self, btnName):
        return not self.joyBtns[btnName].prevDown and \
               self.joyBtns[btnName].down

    def update(self, eventsList):
        # update previous key
        for keyName, key in self.keys.items():
            key.prevDown = key.down

        # update previous joy button
        for btnName, btn in self.joyBtns.items():
            btn.prevDown = btn.down

        # update previous mouse
        self.prevMouse0Down = self.mouse0Down

        for event in eventsList:
            # mouse cursor
            if event.Type == sf.Event.MouseMoved:
                self.mouseX = event.MouseMove.X
                self.mouseY = event.MouseMove.Y

            # mouse button down/up
            elif event.Type == sf.Event.MouseButtonPressed:
                if event.MouseButton.Button == 0:
                    self.mouse0Down = True
            elif event.Type == sf.Event.MouseButtonReleased:
                if event.MouseButton.Button == 0:
                    self.mouse0Down = False

            # key down
            elif event.Type == sf.Event.KeyPressed:
                for keyName, key in self.keys.items():
                    if event.Key.Code == key.code:
                        key.down = True
            # key up
            elif event.Type == sf.Event.KeyReleased:
                for keyName, key in self.keys.items():
                    if event.Key.Code == key.code:
                        key.down = False

            # joystick axis
            elif event.Type == sf.Event.JoyMoved:
                if event.JoyMove.Axis == 6:
                    if event.JoyMove.Position < 0.0:
                        self.joyLeft = True
                    elif event.JoyMove.Position > 0.0:
                        self.joyRight = True
                    else:
                        self.joyLeft = False
                        self.joyRight = False

            # joystick button down
            elif event.Type == sf.Event.JoyButtonPressed:
                for btnName, btn in self.joyBtns.items():
                    if event.JoyButton.Button == btn.num:
                        btn.down = True

            # joystick button up
            elif event.Type == sf.Event.JoyButtonReleased:
                for btnName, btn in self.joyBtns.items():
                    if event.JoyButton.Button == btn.num:
                        btn.down = False

inputState = InputState([("left",      sf.Key.Left),
                         ("right",     sf.Key.Right),
                         ("up",        sf.Key.Up),
                         ("down",      sf.Key.Down),
                         ("space",     sf.Key.Space),
                         ("esc",       sf.Key.Escape),
                         ("f1",        sf.Key.F1),
                         ("f2",        sf.Key.F2),
                         ("f3",        sf.Key.F3),
                         ("f4",        sf.Key.F4),
                         ("f5",        sf.Key.F5),
                         ("c",         sf.Key.C),
                         ("x",         sf.Key.X),
                         ("z",         sf.Key.Z),
                         ("w",         sf.Key.W),
                         ("a",         sf.Key.A),
                         ("s",         sf.Key.S),
                         ("d",         sf.Key.D)],
                        [("jump",   0),
                         ("attack", 2),
                         ("dash",   1),
                         ("menu",   6)])

def inputUpdateTasklet():
    while 1:
        eventsList = channels.inputUpdateChannel.receive()
        
        inputState.update(eventsList)

        channels.broadcast(channels.inputChannel, inputState)
